/*
HDD Explorer Script Generator 
Copyright (c) 2006 Bruno Freitas
bootsector@ig.com.br
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

import java.util.Collection;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class FrmManAplicacao extends JDialog implements ActionListener {
    
    private JPanel pnlDadosAplicacao, pnlArquivosAplicacao, pnlRodape, pnlBtnArquivos, pnlExecutavel, pnlGrupoBtnExecutavel, pnlOkFechar;
    private JLabel lblTitulo, lblAutor, lblDescricao, lblExecutavel, lblDirInst;
    private JTextField txtTitulo, txtAutor, txtDescricao, txtDirInst;
    private JList lstArquivos;
    private DefaultListModel lmArquivos;
    private JComboBox cbExecutavel;
    private JButton btnArqAdiciona, btnArqRemove, btnArqAdPasta, btnArqLimpa, btnOk, btnFechar;
    private Aplicacao aplicacao;
    private boolean incluir;
    
    public FrmManAplicacao(Frame owner, String titulo, Aplicacao aplicacao, boolean incluir) {
        super(owner, titulo, true);
        
        this.aplicacao = aplicacao;
        this.incluir = incluir;
        
        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        
        // Painel Dados da Aplicao
        pnlDadosAplicacao = new JPanel(new GridLayout(6, 1));
        pnlDadosAplicacao.setBorder(new TitledBorder("Dados da Aplicao:"));
        
        lblTitulo = new JLabel("Ttulo:");
        pnlDadosAplicacao.add(lblTitulo);
        txtTitulo = new JTextField(10);
        txtTitulo.setText(aplicacao.getTitulo());
        pnlDadosAplicacao.add(txtTitulo);
        
        lblAutor = new JLabel("Autor:");
        pnlDadosAplicacao.add(lblAutor);
        txtAutor = new JTextField(10);
        txtAutor.setText(aplicacao.getAutor());
        pnlDadosAplicacao.add(txtAutor);
        
        lblDescricao = new JLabel("Descrio (utilize \\n para quebrar a linha):");
        pnlDadosAplicacao.add(lblDescricao);
        txtDescricao = new JTextField(10);
        txtDescricao.setText(aplicacao.getDescricao().replace("\n", "\\n"));
        pnlDadosAplicacao.add(txtDescricao);
        
        // Painel Arquivos da Aplicao
        pnlArquivosAplicacao = new JPanel(new BorderLayout());
        pnlArquivosAplicacao.setBorder(new TitledBorder("Arquivos:"));
        
        lmArquivos = new DefaultListModel();
        lstArquivos = new JList(lmArquivos);
        
        pnlBtnArquivos = new JPanel(new GridLayout(1, 4));
        
        btnArqAdiciona = new JButton("Adicionar");
        btnArqAdiciona.setMnemonic('A');
        btnArqAdiciona.setIcon(new ImageIcon(getClass().getResource("res/img/novo.png")));
        btnArqAdiciona.addActionListener(this);
        pnlBtnArquivos.add(btnArqAdiciona);
        
        btnArqAdPasta = new JButton("Adicionar Pasta");
        btnArqAdPasta.setMnemonic('P');
        btnArqAdPasta.setIcon(new ImageIcon(getClass().getResource("res/img/pasta.png")));
        btnArqAdPasta.addActionListener(this);
        pnlBtnArquivos.add(btnArqAdPasta);
        
        btnArqRemove = new JButton("Remover");
        btnArqRemove.setMnemonic('R');
        btnArqRemove.setIcon(new ImageIcon(getClass().getResource("res/img/deletar.png")));
        btnArqRemove.addActionListener(this);
        pnlBtnArquivos.add(btnArqRemove);
        
        btnArqLimpa = new JButton("Limpar Lista");
        btnArqLimpa.setMnemonic('L');
        btnArqLimpa.setIcon(new ImageIcon(getClass().getResource("res/img/limpa.png")));
        btnArqLimpa.addActionListener(this);
        pnlBtnArquivos.add(btnArqLimpa);
        
        pnlExecutavel = new JPanel(new GridLayout(2, 1));
        lblExecutavel = new JLabel("Arquivo Executvel:");
        pnlExecutavel.add(lblExecutavel);
        cbExecutavel = new JComboBox();
        pnlExecutavel.add(cbExecutavel);
        
        pnlGrupoBtnExecutavel = new JPanel(new BorderLayout());
        pnlGrupoBtnExecutavel.add(pnlBtnArquivos, BorderLayout.NORTH);
        pnlGrupoBtnExecutavel.add(pnlExecutavel, BorderLayout.SOUTH);
        
        pnlArquivosAplicacao.add(new JScrollPane(lstArquivos), BorderLayout.CENTER);
        pnlArquivosAplicacao.add(pnlGrupoBtnExecutavel, BorderLayout.SOUTH);
        
        // Painel Rodap
        pnlRodape = new JPanel(new GridLayout(3, 1));
        pnlRodape.setBorder(new TitledBorder("Dados da Instalao:"));
        
        lblDirInst = new JLabel("Diretrio da instalao (apenas o nome do diretrio):");
        pnlRodape.add(lblDirInst);
        txtDirInst = new JTextField(10);
        txtDirInst.setText(aplicacao.getDiretorio());
        pnlRodape.add(txtDirInst);
        
        pnlOkFechar = new JPanel(new GridLayout(1, 2));
        btnOk = new JButton("OK");
        btnOk.setMnemonic('O');
        btnOk.setIcon(new ImageIcon(getClass().getResource("res/img/ok.png")));
        btnOk.addActionListener(this);
        pnlOkFechar.add(btnOk);
        
        btnFechar = new JButton("Fechar");
        btnFechar.setMnemonic('F');
        btnFechar.setIcon(new ImageIcon(getClass().getResource("res/img/cancelar.png")));
        btnFechar.addActionListener(this);
        pnlOkFechar.add(btnFechar);
        
        pnlRodape.add(pnlOkFechar);
        
        container.add(pnlDadosAplicacao, BorderLayout.NORTH);
        container.add(pnlArquivosAplicacao, BorderLayout.CENTER);
        container.add(pnlRodape, BorderLayout.SOUTH);
        
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        
        setSize(550, 500);
        setResizable(false);
        setLocationRelativeTo(owner);
        
        atualizaListaArquivos();
    }
    
    private boolean validaDirInst() {
        char[] proibidos = {'\\', ' ', '/'};
        String s = txtDirInst.getText();
        
        for (int i = 0; i < s.length(); i++) {
            for (int j = 0; j < proibidos.length; j++) {
                if (s.charAt(i) == proibidos[j]) {
                    return false;
                }
            }
        }
        
        return true;
    }
    
    private void atualizaListaArquivos() {
        lmArquivos.removeAllElements();
        
        for (int i = 0; i < aplicacao.getTotalArquivos(); i++) {
            lmArquivos.addElement(aplicacao.getArquivo(i));
        }
        
        atualizaComboBoxExecutavel();
    }
    
    private void atualizaComboBoxExecutavel() {
        cbExecutavel.removeAllItems();
        
        for (int i = 0; i < aplicacao.getTotalArquivos(); i++) {
            cbExecutavel.addItem(aplicacao.getArquivo(i).substring(aplicacao.getArquivo(i).lastIndexOf(File.separatorChar)+1));
        }
        
        if (aplicacao.getExecutavel().compareTo("") != 0) {
            // Localiza indice do executavel na lista e coloca como padro
            int i;
            boolean achou = false;
            
            for(i = 0; i < cbExecutavel.getItemCount(); i++) {
                String s = (String) cbExecutavel.getItemAt(i);
                if (aplicacao.getExecutavel().compareToIgnoreCase(s) == 0) {
                    cbExecutavel.setSelectedIndex(i);
                    achou = true;
                    break;
                }
            }
            
            if (!achou) {
                if (aplicacao.getTotalArquivos() == 0)
                    aplicacao.setExecutavel("");
                else
                    aplicacao.setExecutavel((String)cbExecutavel.getSelectedItem());
            }
        }
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btnFechar) {
            if (aplicacao.getTotalArquivos() == 0 && !incluir) {
                JOptionPane.showMessageDialog(this, "Voc dele ter pelo menos um arquivo adicionado na lista!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            if (aplicacao.getTotalArquivos() > 31 && !incluir) {
                JOptionPane.showMessageDialog(this, "O nmero mximo de arquivos na lista  31!\nA lista possui " + aplicacao.getTotalArquivos() + " arquivos.", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            setVisible(false);
        }
        
        if (e.getSource() == btnOk) {
            if (txtTitulo.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "Voc deve informar o ttulo da aplicao!", "Ateno", JOptionPane.WARNING_MESSAGE);
                txtTitulo.requestFocus();
                return;
            }
            
            if (txtAutor.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "Voc deve informar o autor da aplicao!", "Ateno", JOptionPane.WARNING_MESSAGE);
                txtAutor.requestFocus();
                return;
            }
            
            if (txtDescricao.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "Voc deve informar a descrio da aplicao!", "Ateno", JOptionPane.WARNING_MESSAGE);
                txtDescricao.requestFocus();
                return;
            }
            
            if (aplicacao.getTotalArquivos() == 0) {
                JOptionPane.showMessageDialog(this, "Voc deve ter pelo menos um arquivo adicionado na lista!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            if (aplicacao.getTotalArquivos() > 31) {
                JOptionPane.showMessageDialog(this, "O nmero mximo de arquivos na lista  31!\nA lista possui " + aplicacao.getTotalArquivos() + " arquivos.", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }

            if (txtDirInst.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "Voc deve informar o diretrio da aplicao!", "Ateno", JOptionPane.WARNING_MESSAGE);
                txtDirInst.requestFocus();
                return;
            }
            
            if (!validaDirInst()) {
                JOptionPane.showMessageDialog(this, "O diretrio da aplicao contm caracteres invlidos: '\\', ' ' ou '/'!", "Ateno", JOptionPane.WARNING_MESSAGE);
                txtDirInst.requestFocus();
                return;
            }
            
            aplicacao.setTitulo(txtTitulo.getText());
            aplicacao.setAutor(txtAutor.getText());
            aplicacao.setDescricao(txtDescricao.getText());
            aplicacao.setDiretorio(txtDirInst.getText().toUpperCase());
            aplicacao.setExecutavel((String)cbExecutavel.getSelectedItem());
            setVisible(false);
        }
        
        
        if (e.getSource() == btnArqAdiciona) {
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setDialogTitle("Adicionar arquivo...");
                
                if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                    aplicacao.adicionaArquivo(fc.getSelectedFile().getAbsolutePath());
                    atualizaListaArquivos();
                }
            } catch (Exception ex) {
                System.err.println(ex);
            }
        }
        
        if (e.getSource() == btnArqRemove) {
            if (lmArquivos.size() == 0) {
                JOptionPane.showMessageDialog(this, "No existem arquivos para serem excludos!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            String arquivo = (String) lstArquivos.getSelectedValue();
            
            if (arquivo == null) {
                JOptionPane.showMessageDialog(this, "Selecione um arquivo da lista primeiro!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            Object[] opcoes = {"Sim", "No"};
            if (JOptionPane.showOptionDialog(this, "Confirma excluso do arquivo selecionado?", "Confirmao", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, opcoes, opcoes[1]) == JOptionPane.YES_OPTION) {
                aplicacao.removeArquivo(lstArquivos.getSelectedIndex());
                atualizaListaArquivos();
            }
        }
        
        if (e.getSource() == btnArqLimpa) {
            Object[] opcoes = {"Sim", "No"};
            if (JOptionPane.showOptionDialog(this, "Confirma excluso de TODOS os arquivos da lista?", "Confirmao", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, opcoes, opcoes[1]) == JOptionPane.YES_OPTION) {
                for (int i = aplicacao.getTotalArquivos()-1; i >= 0; i--) {
                    aplicacao.removeArquivo(i);
                }
                atualizaListaArquivos();
            }
        }
        
        if (e.getSource() == btnArqAdPasta) {
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setDialogTitle("Adicionar diretorio...");
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                
                if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                    File diretorio = new File(fc.getSelectedFile().getAbsolutePath());
                    File arquivos[];
                    FileFilter filtro = new FileFilter() {
                        public boolean accept(File file) {
                            return (file.isFile());
                        }
                    };
                    
                    arquivos = diretorio.listFiles(filtro);
                    
                    for (int i = 0; i < arquivos.length; i++) {
                        aplicacao.adicionaArquivo(arquivos[i].getAbsoluteFile().toString());
                    }
                    
                    atualizaListaArquivos();
                }
            } catch (Exception ex) {
                System.err.println(ex);
            }
        }
    }
}
